/*****************************************************************************
 * LinuxDVD.h: Linux-like DVD driver for Darwin and MacOS X
 *****************************************************************************
 * Copyright (C) 1998-2000 Apple Computer, Inc. All rights reserved.
 * Copyright (C) 2001 VideoLAN
 * $Id: LinuxDVD.h sam Exp $
 *
 * Authors: Samuel Hocevar <sam@zoy.org>
 *
 * The contents of this file constitute Original Code as defined in and
 * are subject to the Apple Public Source License Version 1.1 (the
 * "License").  You may not use this file except in compliance with the
 * License.  Please obtain a copy of the License at
 * http://www.apple.com/publicsource and read it before using this file.
 * 
 * This Original Code and all software distributed under the License are
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
 * License for the specific language governing rights and limitations
 * under the License.
 *****************************************************************************/

class LinuxDVD : public IOService
{
    OSDeclareDefaultStructors( LinuxDVD )

public:

    virtual bool       init   ( OSDictionary *dictionary = 0 );
    virtual IOService *probe  ( IOService *provider, SInt32 *score );
    virtual bool       start  ( IOService *provider );
    virtual void       stop   ( IOService *provider );
    virtual void       free   ( void );
};

#define super IOService
OSDefineMetaClassAndStructors( LinuxDVD, IOService )

struct sum { int a, b, r; };
#define IODVD_READ_STRUCTURE _IOWR('B', 1, struct sum)
#define IODVD_SEND_KEY       _IOWR('B', 2, struct sum)
#define IODVD_REPORT_KEY     _IOWR('B', 3, struct sum)

#define get_kernel_task() kernel_task
#define get_user_task()   current_task()

typedef void * dkr_t;
typedef enum { DKRTYPE_BUF, DKRTYPE_DIO } dkrtype_t;
struct dio { dev_t dev; struct uio * uio; };

/*****************************************************************************
 * Local variables
 *****************************************************************************/
static LinuxDVD * p_this = NULL;

static bool b_inuse;
static int i_major;
static void *p_node;
static IODVDMedia *p_dvd;

/*****************************************************************************
 * DKR_GET_DEV: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
inline dev_t DKR_GET_DEV(dkr_t dkr, dkrtype_t dkrtype)
{
    return (dkrtype == DKRTYPE_BUF)
           ? ((struct buf *)dkr)->b_dev
           : ((struct dio *)dkr)->dev;
}

/*****************************************************************************
 * DKR_GET_BYTE_COUNT: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
static inline UInt64 DKR_GET_BYTE_COUNT(dkr_t dkr, dkrtype_t dkrtype)
{
    return (dkrtype == DKRTYPE_BUF)
           ? ((struct buf *)dkr)->b_bcount
           : ((struct dio *)dkr)->uio->uio_resid;
}

/*****************************************************************************
 * DKR_GET_BYTE_START: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
static inline UInt64 DKR_GET_BYTE_START(dkr_t dkr, dkrtype_t dkrtype)
{
    if (dkrtype == DKRTYPE_BUF)
    {
        struct buf * bp    = (struct buf *)dkr;
        return bp->b_blkno * p_dvd->getPreferredBlockSize();
    }
    return ((struct dio *)dkr)->uio->uio_offset;
}

/*****************************************************************************
 * DKR_IS_READ: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
static inline bool DKR_IS_READ(dkr_t dkr, dkrtype_t dkrtype)
{
    return (dkrtype == DKRTYPE_BUF)
           ? ((((struct buf *)dkr)->b_flags & B_READ) == B_READ)
           : ((((struct dio *)dkr)->uio->uio_rw) == UIO_READ);
}

/*****************************************************************************
 * DKR_IS_ASYNCHRONOUS: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
static inline bool DKR_IS_ASYNCHRONOUS(dkr_t dkr, dkrtype_t dkrtype)
{
    return (dkrtype == DKRTYPE_BUF)
           ? true
           : false;
}

/*****************************************************************************
 * DKR_IS_RAW: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
static inline bool DKR_IS_RAW(dkr_t dkr, dkrtype_t dkrtype)
{
    return (dkrtype == DKRTYPE_BUF)
           ? false
           : true;
}

/*****************************************************************************
 * DKR_SET_BYTE_COUNT: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
static inline void DKR_SET_BYTE_COUNT(dkr_t dkr, dkrtype_t dkrtype, UInt64 bcount)
{
    if (dkrtype == DKRTYPE_BUF)
        ((struct buf *)dkr)->b_resid = ((struct buf *)dkr)->b_bcount - bcount;
    else
        ((struct dio *)dkr)->uio->uio_resid -= bcount;
}

/*****************************************************************************
 * DKR_RUN_COMPLETION: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
static inline void DKR_RUN_COMPLETION(dkr_t dkr, dkrtype_t dkrtype, IOReturn status)
{
    if (dkrtype == DKRTYPE_BUF)
    {
        struct buf * bp = (struct buf *)dkr;

        bp->b_error  = p_this->errnoFromReturn(status);
        bp->b_flags |= (status != kIOReturnSuccess) ? B_ERROR : 0;
        biodone(bp);
    }
}

/*****************************************************************************
 * DKR_GET_BUFFER: borrowed from IOMediaBSDClient.cpp
 *****************************************************************************/
static inline IOMemoryDescriptor * DKR_GET_BUFFER(dkr_t dkr, dkrtype_t dkrtype)
{
    if (dkrtype == DKRTYPE_BUF)
    {
        struct buf * bp = (struct buf *)dkr;

        if ( (bp->b_flags & B_VECTORLIST) )
        {
            assert(sizeof(IOPhysicalRange         ) == sizeof(iovec          ));
            assert(sizeof(IOPhysicalRange::address) == sizeof(iovec::iov_base));
            assert(sizeof(IOPhysicalRange::length ) == sizeof(iovec::iov_len ));
            return IOMemoryDescriptor::withPhysicalRanges(
              (IOPhysicalRange *) bp->b_vectorlist,
              (UInt32)            bp->b_vectorcount,
              (bp->b_flags & B_READ) ? kIODirectionIn : kIODirectionOut,
              true );
        }

        return IOMemoryDescriptor::withAddress(
          (vm_address_t) bp->b_data,
          (vm_size_t)    bp->b_bcount,
          (bp->b_flags & B_READ) ? kIODirectionIn : kIODirectionOut,
          (bp->b_flags & B_PHYS) ? get_user_task() : get_kernel_task() );
    }
    else
    {
        struct uio * uio = ((struct dio *)dkr)->uio;

        assert(sizeof(IOVirtualRange         ) == sizeof(iovec          ));
        assert(sizeof(IOVirtualRange::address) == sizeof(iovec::iov_base));
        assert(sizeof(IOVirtualRange::length ) == sizeof(iovec::iov_len ));

        return IOMemoryDescriptor::withRanges(
        (IOVirtualRange *) uio->uio_iov,
        (UInt32)           uio->uio_iovcnt,
        (uio->uio_rw     == UIO_READ    ) ? kIODirectionIn  : kIODirectionOut,
        (uio->uio_segflg != UIO_SYSSPACE) ? get_user_task() : get_kernel_task(),        true );
    }
}

